part of 'package:web3dart_go_core/crypto.dart';

/// If present, removes the 0x from the start of a hex-string.
String strip0xOrPrefix(String hex) {
  if (hex.startsWith('0x')) return hex.substring(2);
  // if (hex.startsWith('ce')) return hex.substring(2);
  // if (hex.startsWith('cb')) return hex.substring(2);
  // if (hex.startsWith('ab')) return hex.substring(2);
  return hex;
}

/// Converts the [bytes] given as a list of integers into a hexadecimal
/// representation.
///
/// If any of the bytes is outside of the range [0, 256], the method will throw.
/// The outcome of this function will prefix a 0 if it would otherwise not be
/// of even length. If [include0x] is set, it will prefix "0x" to the hexadecimal
/// representation. If [forcePadLength] is set, the hexadecimal representation
/// will be expanded with zeroes until the desired length is reached. The "0x"
/// prefix does not count for the length.
String bytesToHex(List<int> bytes,
    {bool include0x = false,
    int forcePadLength,
    bool padToEvenLength = false,
    bool calledFromTransaction = false}) {
  var encoded = hex.encode(bytes);
  // if (forcePadLength == 44) {
  //   final padding = forcePadLength - encoded.length;
  //   encoded = ('0' * (padding - 2)) + encoded;
  //   encoded = 'ce$encoded';
  // }
  if (padToEvenLength && encoded.length % 2 != 0) {
    encoded = '0$encoded';
  }
  if (forcePadLength == null && include0x) {
    encoded = '0x$encoded';
  }

  return encoded;
}

/// Converts the hexadecimal string, which can be prefixed with 0x, to a byte
/// sequence.
Uint8List hexToBytes(String hexStr) {
  final bytes = hex.decode(strip0xOrPrefix(hexStr));
  if (bytes is Uint8List) return bytes;

  return Uint8List.fromList(bytes);
}

///Converts the bytes from that list (big endian) to an unsigned BigInt.
BigInt bytesToInt(List<int> bytes) => p_utils.decodeBigInt(bytes);

Uint8List intToBytes(BigInt number) => p_utils.encodeBigInt(number);

///Takes the hexadecimal input and creates a [BigInt].
BigInt hexToInt(String hex) {
  return BigInt.parse(strip0xOrPrefix(hex), radix: 16);
}

/// Converts the hexadecimal input and creates an [int].
int hexToDartInt(String hex) {
  return int.parse(strip0xOrPrefix(hex), radix: 16);
}
