part of 'package:web3dart_go_core/whisper.dart';

class FilteredMessagesModel {
  ///* sig - String: Public key who signed this message.
  ///* ttl - Number: Time-to-live in seconds.
  ///* timestamp - Number: Unix timestamp of the message generation.
  ///* topic - String 4 Bytes: Message topic.
  ///* payload - String: Decrypted payload.
  ///* padding - String: Optional padding (byte array of arbitrary length).
  ///* pow - Number: Proof of work value.
  ///* hash - String: Hash of the enveloped message.
  ///* recipientPublicKey - String: recipient public key
  String _sig;
  int _ttl;
  int _timestamp;
  String _topic;
  String _payload;
  String _padding;
  double _pow;
  String _hash;
  String _recipientPublicKey;
  FilteredMessagesModel.fromJSON(Map<String, dynamic> json) {
    _sig = json['sig'].toString() ?? '';
    _ttl = int.tryParse(json['ttl'].toString()) ?? 7;
    _timestamp = int.tryParse(json['timestamp'].toString()) ?? 1000;
    _topic = json['_topic'].toString() ?? '';
    _payload = json['payload'].toString() ?? '';
    _padding = json['padding'].toString() ?? '';
    _pow = double.tryParse(json['pow'].toString()) ?? 2.01;
    _hash = json['hash'].toString() ?? '';
    _recipientPublicKey = json['recipientPublicKey'].toString() ?? '';
  }
  String get sig => _sig;
  int get ttl => _ttl;
  int get timestamp => _timestamp;
  String get topic => _topic;
  String get payload => _payload;
  String get padding => _padding;
  double get pow => _pow;
  String get hash => _hash;
  String get recipientPublicKey => _recipientPublicKey;
}

class WhisperMessage {
  Uint8List _msg;
  WhisperMessage.fromHex(String hexPayload) {
    final _tmp = strip0xOrPrefix(hexPayload);
    final _kec = sha3Ascii(_tmp);
    final hex = bytesToHex(_kec);
    _msg = hexToBytes(hex);
  }
  Uint8List get message => _msg;
}
